
/*****************************************************************************
 * File:			main.c
 * Description:		Main application file
 * Project:			Intrusion system and access control
 * 					(Sistema de intrusión y control de accesos)
 *
 * Version:			1.0		02/01/2017
 * Author:			Daniel Gómez
 *****************************************************************************/

/* FreeRTOS Includes */
#include "FreeRTOS.h"
#include "task.h"

/* LPC Includes */
#include "LPC17xx.h"
#include "type.h"

/* Library Includes */
#include "gpio.h"
#include "serial.h"
#include "wifly.h"
#include "log.h"
#include "stdio.h"
#include "utils.h"

/* App Includes */
#include "alarm.h"
#include "mifare.h"


#define configTYP_STACK_SIZE 1024
#define MILIS_TO_TICKS(ms) (ms/portTICK_RATE_MS)
#define PERIOD_TIME 0.1

char buff3Tx[1000]; // buffer to receive the response of the Driver through UART


/* Task functions */
static void checkPeripherals( void *pvParameters );
static void accessControl( void *pvParameters );

/* Variable */
int armed=1;

/* Access Point Configuration */
static char* WLAN_SSID = "vodafone6150";		//network SSID
static char* WLAN_PWD = "MUAGKFDKKTDYVG";		//network password

/* TCP Server */
//static char *ServerHost = "tfc.sergiogomez.es";	//Host Name or IP (ex: 'tfc.sergiogomez.es' or '192.168.1.18')
//static int ServerPort = 6001;					//Port
//static uint8_t HostType = HOSTDNS;				//HOSTDNS for Name or HOSTIP for IP

/*****************************************************************************
 * Function:		main
 * Description:		Initializes peripherals, creates queues and tasks,
 * 					and set in motion planner.
 *
 * @return			None
 *****************************************************************************/

int main(void) {

	/*Initializes log, wifly, alarm and mifare */
	Log_start(SERIAL_PORT1, BAUD_115200);
	Wifly_init(3,57600);
	alarmInit();
	mifareInit();

	/* Connects wifly to the SSID */
	WiflyConect(WLAN_PWD,WLAN_SSID);

	/*Task creator*/
	xTaskCreate( checkPeripherals,		( signed char * ) "CheckPeripherals",  configTYP_STACK_SIZE, NULL, tskIDLE_PRIORITY+1, NULL );
	xTaskCreate( accessControl,			( signed char * ) "accessControl", 		configTYP_STACK_SIZE, NULL, tskIDLE_PRIORITY+1, NULL );

	/* Start the tasks running. */
	vTaskStartScheduler();


	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */


	for( ;; );
}
/*-----------------------------------------------------------*/

	/*External Interrupt for MIFARE Reader Data0 */
void EINT0_IRQHandler (void)
{
	NVIC_DisableIRQ(EINT0_IRQn);	//disables EINT0 in NVIC

	if( ((LPC_GPIOINT->IO2IntStatF)& (1<<10)) ) //EINT0 falling edge
	{
		data0();  						//calls mifare function
		LPC_SC->EXTINT &= 1;        	//removes interruption
	}
	NVIC_EnableIRQ(EINT0_IRQn);			//enables EINT0 in NVIC
}



	/*External Interrupt for MIFARE Reader Data1 */
void EINT1_IRQHandler(void)
{
	NVIC_DisableIRQ(EINT1_IRQn);		//disables EINT1 in NVIC

	if( ((LPC_GPIOINT->IO2IntStatF)& (1<<11)) ) //EINT1 falling edge
    {
		data1();						//calls mifare function
		LPC_SC->EXTINT &= 1<<1;        	//removes interruption
    }
	NVIC_EnableIRQ(EINT1_IRQn);			//enables EINT1 in NVIC
}



	/*External Interrupt for the key switch*/
void EINT2_IRQHandler (void)
{
	NVIC_DisableIRQ(EINT2_IRQn);		//disables EINT2 in NVIC

	if( ((LPC_GPIOINT->IO2IntStatF)& (1<<12)) ) //EINT2 falling edge
	{
		if(armed==1)		//If it is armed: disarm, else:arm
		{
			armed=0;
		}
		else{
			armed=1;
		}
		LPC_SC->EXTINT &= 1<<2;     //removes interruption
	}
	NVIC_EnableIRQ(EINT2_IRQn);		//enables EINT2 in NVIC
}

/*****************************************************************************
 *                                  TASKS
 *****************************************************************************/
/*****************************************************************************
 * Task Name:		checkPeripherals
 * Description:		Monitors the status of the intrusion system, GPIO pins with entries
 * 					(volumethric detector,magnethic contact) in order to check if there is
 * 					an intrussion and consequently, activate the alarm.
 *
 * @return			None
 *****************************************************************************/

static void checkPeripherals( void *pvParameters ) {
	int reposeStatus=0;


	for( ;; ) {

		vTaskDelay(MILIS_TO_TICKS(PERIOD_TIME));

		if(armed==0)					//if status is armed
		{
			Gpio_set(0,0);				//status green LED
			Gpio_set(6,1);				//status red LED
			Disarmed();					//mifare function
			alarmOff();					//alarm function
			vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME));
		}
		else{							//if status is disarmed
			Gpio_set(6,0);				//status red LED
			Gpio_set(0,1);				//status green LED
			Armed();					//mifare function
			vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME));
			int magnethicContact=GPIO_getValue(1);	//magnethic status
			int volumetricDetector=GPIO_getValue(2);//volumethric status
			if ((magnethicContact==1)&&(volumetricDetector==1)) 	//if status is repose
			{
				if(reposeStatus==0)
				{
					Log_print("Reposo\r");
					reposeStatus=1;
				}
				vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME));
			}
			else if((magnethicContact!=1)&&(volumetricDetector==1))	//magnethic detection
			{
				alarmOn(MAGNETHIC);					//alarm function
				reposeStatus=0;
				vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME));
			}
			else if((magnethicContact==1)&&(volumetricDetector!=1))	//volumethric detection
			{
				alarmOn(VOLUMETHRIC);				//alarm function
				reposeStatus=0;
				vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME));
			}
			else
			{
				alarmOn(BOTH);						//alarm function
				reposeStatus=0;
				vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME));
			}
		}
	}

}


/*****************************************************************************
 * Task Name:		accessControl
 * Description:		Controls the MIFARE reader in order to accept or deny a person's access,
 * 					arm or disarm the intrussion system and opens the door
 *
 * @return			None
 *****************************************************************************/
static void accessControl( void *pvParameters ) {

	for( ;; ) {

		vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME));
		cardReader();							//mifare function
		if(acceptedID()==1){
			if(armed==0){
				armed=1;
				sprintf(buff3Tx,"El sistema de intrusión ha sido armado por el usuario %d\r", getCardCode());
				wiflyEmail("dungeon713@hotmail.com", "INTRUSIÓN MODO ARMADO", buff3Tx);
				resetCardcode();
			}else{
				armed=0;
				Gpio_set(9,1);
				Utils_delayMs(2000);
				Gpio_set(9,0);
				sprintf(buff3Tx,"El sistema de intrusión ha sido desarmado por el usuario %d\r", getCardCode());
				wiflyEmail("dungeon713@hotmail.com", "INTRUSIÓN MODO DESARMADO", buff3Tx);
				resetCardcode();
			}
		}
		vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME));
	}
}

/*****************************************************************************
 *                             HOOK FUNCTIONS
 *****************************************************************************/


void vApplicationMallocFailedHook( void ) {
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/


void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName ) {
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	Log_print("[ERROR] Stack Overflow\r");
	Log_print(pcTaskName);
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void ) {
	/* This project does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void ) {
	/* This projects does not use the tick hook to perform any processing. */
}
