/*****************************************************************************
 * File:			wifly.c
 * Description:		Wifly driver to communicate with wifly module
 *
 * Author:			Daniel Gómez
 * Created:			02/01/2017
 *****************************************************************************/

#include "assertlib.h"
#include "LPC17xx.h"
#include "type.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "log.h"
#include "string.h"
#include "wifly.h"
#include "serial.h"
#include "gpio.h"
#include "utils.h"
#include "uart.h"
#include "stdio.h"



xSemaphoreHandle wiflyMux;

uint32_t sizeAsk; // stores the amount of characters received as response of comands sent to driver


char buffRx[UART_RX_BUFFER]; // buffer to receive the response of the Driver through UART
char buffTx[UART_RX_BUFFER]; // buffer to send comand to the Driver through UART


char str_ln[3] = {0x0D, 0x0A, 0x00}; // char type constant to add a line break in hexadecimal

/*
 * ----------------------------------------
 * -------------METHODS -------------------
 * ----------------------------------------
 */

static void WiflyCleanBuffers (){					//clean buffRx and buffTx buffers

	 memset(buffRx, 0x00, UART_RX_BUFFER);
     memset(buffTx, 0x00, UART_RX_BUFFER);
}


static void _resetWifly() {							//resets wifly
	Gpio_config(WIFLY_RESET_PAD, GPIO_OUT);
	Gpio_set(WIFLY_RESET_PAD, GPIO_HIGH);
	Utils_delayMs(1000);	// min 160us
	Gpio_set(WIFLY_RESET_PAD, GPIO_LOW);
	Utils_delayMs(1000);	// min 160us
	Gpio_set(WIFLY_RESET_PAD, GPIO_HIGH);
}

void Wifly_init(uint8_t port, serial_baudrate baudRate) {		//Configure UART Port to communicate with wifly

	if (wiflyMux == NULL) wiflyMux = xSemaphoreCreateRecursiveMutex();		//creates a recursive mutex in order to recatch the semaphore inside the same semaphore
	_resetWifly();
	UART_init(port,baudRate);
	Utils_delayMs(3000);
	Log_print("Iniciando Wifly\r");
	WiflyCleanBuffers();
	UART_Scanf(3,buffRx,3000);
	if(buffRx!=NULL)Log_print(buffRx);
}

uint32_t WiflyComandLog (int UartPort, char *Comando){		//send commands to wifly and prints the response on log

	xSemaphoreTakeRecursive( wiflyMux, portMAX_DELAY );
		{
		sprintf(buffTx, "%s\r\n", Comando);
		UART_Send(UartPort,(uint8_t *)buffTx,strlen(buffTx));
		WiflyResponse(1000);
		WiflyCleanBuffers();
		}
	xSemaphoreGiveRecursive( wiflyMux );

	return 0;
}

uint32_t WiflyComand (int UartPort, char *Comando){			//send commands to wifly without printing response on log

	xSemaphoreTakeRecursive( wiflyMux, portMAX_DELAY );
		{
		sprintf(buffTx, "%s\r\n", Comando);
		UART_Send(UartPort,(uint8_t *)buffTx,strlen(buffTx));
		WiflyCleanBuffers();
		}
	xSemaphoreGiveRecursive( wiflyMux );

	return 0;
}

uint32_t WiflyConsola (int UartPort){				//opens the console in wifly sending $$$

	xSemaphoreTakeRecursive( wiflyMux, portMAX_DELAY );
	{
	sprintf(buffTx,"$$$");
	UART_Send(UartPort,(uint8_t *)buffTx,strlen(buffTx));
	WiflyResponse(1000);
	WiflyCleanBuffers();
	}
	xSemaphoreGiveRecursive( wiflyMux );

	return 0;
}

uint32_t WiflyConect (char *Password, char *Red){	//connects wifly to the given SSID


	WiflyConsola(3);
	sprintf(buffTx,"set wlan phrase %s",Password);
	WiflyComand(3,buffTx);
	sprintf(buffTx,"join %s",Red);
	WiflyComandLog(3,buffTx);

	return 0;

}

uint32_t WiflyResponse (int TiempoEspera){					//command to print response of wifly in log

	sizeAsk=UART_Scanf(3,buffRx,TiempoEspera);
	Log_print("LOG WiFly: Hemos recibido %d chars con: %s \r", sizeAsk, buffRx);

	return 0;
}

uint8_t wiflyEmail(char *Receiver, char *Subject, char *Data){		//sends an e-mail with the given parameters


	xSemaphoreTakeRecursive( wiflyMux, portMAX_DELAY );
	wiflySetTCPConfig();
		{
			WiflyComand(3,"open");
			Utils_delayMs(3000);
			WiflyComand(3,"EHLO");
			WiflyComand(3,"auth login");
			WiflyComand(3,"ZGFuaWdvbWdhcg==");
			WiflyComand(3,"MDI0Njgw");
			WiflyComand(3,"mail from: danigomgar@gmail.com");
			sprintf(buffTx,"rcpt to: %s", Receiver);
			WiflyComand(3,buffTx);
			WiflyComand(3,"data");
			WiflyComand(3,"From: Central de intrusion");
			sprintf(buffTx,"Subject: %s", Subject);
			WiflyComand(3,buffTx);
			sprintf(buffTx,"%s", Data);
			WiflyComand(3,buffTx);
			sprintf(buffTx,".");
			WiflyComandLog(3,buffTx);
			WiflyComand(3,"quit");

		}
		xSemaphoreGiveRecursive( wiflyMux );

		return 0;

}

void wiflySetTCPConfig(){										//sets the config on wifly in order to establish a TCP connection with SMTP server

	xSemaphoreTakeRecursive( wiflyMux, portMAX_DELAY );
		{
		WiflyConsola(3);
		sprintf(buffTx,"set ip proto 2");
		WiflyComand(3,buffTx);
		sprintf(buffTx,"set dns name mail.smtp2go.com");
		WiflyComand(3,buffTx);
		sprintf(buffTx,"set ip host 0");
		WiflyComand(3,buffTx);
		sprintf(buffTx,"set ip remote 2525");
		WiflyComand(3,buffTx);
		sprintf(buffTx,"set com remote 0");
		WiflyComand(3,buffTx);

		}

	xSemaphoreGiveRecursive( wiflyMux );

}




